import kaybee.plugins.articles
import kaybee.plugins.debugdumper.handlers
import kaybee.plugins.genericpage.handlers
import kaybee.plugins.localtemplates.handlers
import kaybee.plugins.queries.handlers
import kaybee.plugins.references.handlers
import kaybee.plugins.references.reference
import kaybee.plugins.resources.handlers
import kaybee.plugins.resources.resource
import kaybee.plugins.settings.handlers
import kaybee.plugins.sphinx_app.handlers
import kaybee.plugins.widgets.handlers
import kaybee.plugins.widgets.widget
